% GMM estimator of CEV model with 3 parameters
%
%
% INPUTS:
% y         = univariate time-series of levels
% start     = starting values of 3 parameters
% null      = null hypothesis vector
% vnames    = variable names
% IWM       = string decoding initial weighting matrix. Permissible values:
%               'I' = Identity, 
%               'Z' = Instruments (Z'Z), (default)
%               'C' = Calculate from model parameters,
% SearchDirection = string indicating hessian update method. Permissible values:
%               'sd'    = steepest descent
%               'gn'    = Gauss-Newton
%               'marq'  = Levenberg-Marquardt
%               'dfp'   = Davidon-Fletcher-Powell (default)
%               'bfgs'  = Broyden-Fletcher-Goldfarb-Shano
% SDMatrixType = string indicating type of spectral density matrix. Permissible values 
%               'I'     = Identity
%               'P'     = Plain (like OLS,  e'e.*.Z'Z)
%               'W'     = White (hetero-consistent)
%               'H'     = Hansen (truncated, wts = 1 or 0)
%               'NW'	= Newey-West (Bartlett weights), default
%               'G'     = Gallant (Parzen weights)
%               'AM'	= Andews or Andrews-Monahan 
% wlags     = lags used in the weighting matrix  [default: nobs^(1/3)]
% gmmit     = number of GMM iterations (NaN is iterated)	[default: 2]
% gmmtol    = GMM convergence tolerance (default: 1e-7)
% maxit     = max number of GMM iterations (if gmmit=NaN), [default=25]
% Strim     = control for demeaning of moments. Permissible values
%               0 = none, 
%               1 = demean e (default)
%               2 = demean Z'e
% Slast     = control for terminal spectral density matrix
%               1 to recalc S at final param estimates (default)
%               2 updates W	
% OUTPUTS:
% p         = vector of estimated parameter values
% pse       = standard errors of estimated parameters
% pvals     = pvalues for estimated parameters
% f         = best GMM objective
% J         = chi-square statistics of model fit
% Jpval     = p-value corresponding to J
% Jdf       = degrees of freedom for chi-square distribution of J
% pcov      = covariance matrix of parameters
% mp        = moment p-values
% t         = t-stat for the null hypothesis
% tdf       = degrees of freedom for the null hypothesis
%
function [p,pse,pvals,f,J,Jpval,Jdf,pcov,mp,t,tdf] = cev3_gmm(y,start,null,vnames,...
    IWM,SearchDirection,SDMatrixType,wlags,gmmit,gmmtol,maxit,Strim,Slast)

% input parameter management
if (nargin<13 || numel(Slast)<1)
    Slast=1;
    if (nargin<12 || numel(Strim)<1)
        Strim=1;
        if (nargin<11 || numel(maxit)<1)
            maxit=25;
            if (nargin<10 || numel(gmmtol)<1)
                gmmtol=1e-7;
                if (nargin<9||numel(gmmit)<1)
                    gmmit=2;
                    if (nargin<8||numel(wlags)<1)
                        wlags=20;
                        if (nargin<7||numel(SDMatrixType)<1)
                            SDMatrixType='NW';
                            if (nargin<6||numel(SearchDirection)<1)
                                SearchDirection='dfp';
                            end
                        end
                    end
                end
            end
        end
    end
end
if (nargin<5||numel(IWM)<1)
    IWM='Z';
    if (nargin<4||numel(vnames)<1)
        vnames=strvcat('mu','sigma','gamma');	% Names
        if (nargin<3||numel(null)<1)
            null=[0 0 0]';
            if (nargin<2||numel(start)<1)
                start=[0;std(diff(y));0.5];
            end
        end
    end
end

        
        
% size outputs
T=size(y,1)-1;
dy=diff(y);



% set CEV3 specific things
%Z = [ones(T,1),y(1:T),y(1:T).^2,y(1:T).^3,y(1:T).^4]; % instruments
%Z = [ones(T,1),y(1:T),abs(dy(1:T))]; % instruments
Z = [ones(T,1),y(1:T),y(1:T).^2]; % instruments
%Z = [ones(T,1),y(1:T),abs(dy(1:T)).^2]; % instruments
%Z = [ones(T,1),y(1:T)]; % instruments
gmmopt.W0=IWM;				% Initial Weighting Matrix
gmmopt.infoz.hess = SearchDirection;		% Search Direction
gmmopt.S = SDMatrixType;
gmmopt.infoz.momt='cev3_moments';		% Moment Conditions
gmmopt.infoz.jake='cev3_jacobian';%'numz';%'cev3_jacobian';		% Jacobian of Moments
gmmopt.lags=wlags;	   			% Lags in weighting matrix
gmmopt.vname = vnames;
gmmopt.null = null;
gmmopt.infoz.parms = [NaN NaN NaN]'; % Special field used by cev3_moments
gmmopt.gmmit=gmmit;  
gmmopt.maxit=maxit;
gmmopt.tol=gmmtol;
gmmopt.Strim=Strim;
gmmopt.Slast=Slast;
gmmopt.plot=0;
%gmmopt.aminfo.p=1;
%gmmopt.aminfo.q=1;
%gmmopt.aminfo.vardum=1;

% call GMM
gout=gmm(start,gmmopt,dy,y(1:T),Z);

% assign outputs
p=gout.b;
pse=gout.se;
pvals=gout.pb;
f=gout.f;
J=gout.J;
Jpval=gout.p;
Jdf=gout.df;
pcov=gout.bcov;
mp=gout.mp;
t=gout.t;
tdf=T-2;