% function [h,test,pval,crv0,crv1] = ad_t_test(sample,df,alpha,mctol,NMC)
% Compute Anderson_darling goodness of fit statistics against
% Student null distribution with pre-specified degrees of freedom
% INPUTS:
% sample(1:N,1) = sample
% df            = degrees of freedom parameter
% alpha         = significance level (default 0.05)
% mctol         = (optional) Monte Carlo tolerance
% NMC           = (optional) number of Monte Carlo replications
% OUTPUTS
% h             = hypothesis non-rejection indicator
% test          = test statistics
% pval          = p-value corresponding to test statistics
% crv0          = (left) critical value at significance level alpha
% crv1          = (right) critical value at significance level alpha
function [h,test,pval,crv0,crv1] = ad_t_test(sample,df,alpha,mctol,NMC)

N=size(sample,1);
mu=mean(sample);
s=std(sample)*sqrt((df-2)/df);
fx=tcdf((sort(sample)-mu)/s,df);
% Compute the test statistic of interest
i=1:N;
S = sum((((2*i)-1)/N)*(log(fx)+log(1-fx(N+1-i))));
test=-N-S;
    
if nargin < 2 || isempty(alpha)
   alpha = 0.05;
else
   if ~isscalar(alpha) || ~(0<alpha && alpha<1)
      error('Significance level must be between 0 and 1');
   end
end



if nargin <5 || isempty(NMC);
    if nargin < 4 || isempty(mctol)
        mctol = 10^(floor(log10(alpha/10)));
    else
        if ~isscalar(mctol) || mctol<=0
            error('Invalid Tolerance' );
        end
    end
    NMC=10^(ceil(log10(alpha*(1-alpha)/mctol^2)));
else
    if ~isscalar(NMC) || NMC<=0
        error('Invalid number of Monte Carlo Draws' );
    end
end


ADstatistics=NaN(NMC,1);

for j=1:NMC
    x=trnd(df,N,1);
    x=sort(x);
    mu=mean(x);
    s=std(x)*sqrt((df-2)/df);
    fx=tcdf((x-mu)/s,df);
    S = sum((((2*i)-1)/N)*(log(fx)+log(1-fx(N+1-i))));
    ADstatistics(j) = -N-S;
end

crv0=quantile(ADstatistics,alpha);
crv1=quantile(ADstatistics,1-alpha);
pval = sum(ADstatistics>test)/NMC;
h = (test>crv1);
if sum(ADstatistics>test)<1
    disp(['No exceptions in ',num2str(NMC),' draws']);
end