% function [cv1_low,cv1_up,cv2_low,cv2_up]=critvalue_ad_t_test(N,df,alpha)
% Compute critical values of one-sided and two-sided goodness-of-fit tests
% using Anderson-Darling statistics for Student t null distribution with
% pre-specified degrees of freedom parameter when scale and location are
% themselves estimated
% INPUTS:
% N     = sample size
% df    = degrees of freedom parameter
% alpha = significance level
% OUTPUTS:
% cv1_low = alpha quantile of Monte Carlo distribution of test statistics
% cv1_up  (1-alpha) quantile of Monte Carlo distribution of test statistics
% cv2_low = alpha/2 quantile of Monte Carlo distribution of test statistics
% cv2_up  = 1-alpha/2 quantile of Monte Carlo distribution of test statistics
function [cv1_low,cv1_up,cv2_low,cv2_up]=critvalue_ad_t_test(N,df,alpha)


if nargin < 2 || isempty(alpha)
   alpha = 0.05;
else
   if ~isscalar(alpha) || ~(0<alpha && alpha<1)
      error('Significance level must be between 0 and 1');
   end
end

alpha0=alpha/2;
% required tolerance
mctol = 10^(floor(log10(alpha/10)));
% required number of Monte Carlo draws
NMC=10^(ceil(log10(alpha0*(1-alpha0)/mctol^2)));

ADstatistics=NaN(NMC,1);

for j=1:NMC
    x=trnd(df,N,1);
    %x=randn(N,1);
    x = sort(x);
    mu=mean(x);
    s=std(x)*sqrt((df-2)/df);
    %s=std(x);
    fx=tcdf((x-mu)/s,df);
    %fx=normcdf(x,mu,s);
    i=1:N;
    S = sum((((2*i)-1)/N)*(log(fx)+log(1-fx(N+1-i))));
    ADstatistics(j)=-N-S;
end
cv1_low=quantile(ADstatistics,alpha);
cv1_up=quantile(ADstatistics,1-alpha);
cv2_low=quantile(ADstatistics,alpha0);
cv2_up=quantile(ADstatistics,1-alpha0);