% function [cv1_low,cv1_up,cv2_low,cv2_up]=critvalue_ad_t_test(N,df,alpha)
% Compute critical values of one-sided and two-sided goodness-of-fit tests
% using Kolmogorov-Smirnov-Lilliefors statistics for Student t null distribution with
% pre-specified degrees of freedom parameter when scale and location are
% themselves estimated
% INPUTS:
% N     = sample size
% df    = degrees of freedom parameter
% alpha = significance level (default 0.05)
% OUTPUTS:
% cv1_low = alpha quantile of Monte Carlo distribution of test statistics
% cv1_up  (1-alpha) quantile of Monte Carlo distribution of test statistics
% cv2_low = alpha/2 quantile of Monte Carlo distribution of test statistics
% cv2_up  = 1-alpha/2 quantile of Monte Carlo distribution of test statistics
function [cv1_low,cv1_up,cv2_low,cv2_up]=critvalue_ks_t_test(N,df,alpha)

if nargin < 2 || isempty(alpha)
   alpha = 0.05;
else
   if ~isscalar(alpha) || ~(0<alpha && alpha<1)
      error('Significance level must be between 0 and 1');
   end
end

alpha0=alpha/2;
% required tolerance
mctol = 10^(floor(log10(alpha/10)));
% required number of Monte Carlo draws
NMC=10^(ceil(log10(alpha0*(1-alpha0)/mctol^2)))

KSstatistics=NaN(NMC,1);
if (NMC*N<100000000)
    x0=trnd(df,N,NMC);
    m=mean(x0);
    s0=std(x)*sqrt((df-2)/df);
end

for j=1:NMC
    if (NMC*N<100000000)
        x=x0(:,j);
        mu=m(j);
        s=s0(j);
    else
        x=trnd(df,N,1);
        mu=mean(x);
        s=std(x)*sqrt((df-2)/df);
    end
    [sampleCDF,xCDF]=cdfcalc(x);
    nullCDF=tcdf((xCDF-mu)/s,df);
    % Compute the test statistic of interest: T = max|S(x) - nullCDF(x)|.
    delta1   = sampleCDF(1:end-1) - nullCDF;   % Vertical difference at jumps approaching from the LEFT.
    delta2   = sampleCDF(2:end)   - nullCDF;   % Vertical difference at jumps approaching from the RIGHT.
    deltaCDF = abs([delta1 ; delta2]);

    KSstatistics(j) = max(deltaCDF);
end

cv1_low=quantile(KSstatistics,alpha);
cv1_up=quantile(KSstatistics,1-alpha);
cv2_low=quantile(KSstatistics,alpha0);
cv2_up=quantile(KSstatistics,1-alpha0);