% function [h,test,pval,crv0,crv1] = ks_t_test(sample,df,alpha,mctol,NMC)
% Compute Kolmogorov-Snirnov-Lilliefors goodness of fit statistics against
% Student null distribution with pre-specified degrees of freedom
% INPUTS:
% sample(1:N,1) = sample
% df            = degrees of freedom parameter
% alpha         = significance level (default 0.05)
% mctol         = (optional) Monte Carlo tolerance
% NMC           = (optional) number of Monte Carlo replications
% OUTPUTS
% h             = hypothesis non-rejection indicator
% test          = test statistics
% pval          = p-value corresponding to test statistics
% crv0          = (left) critical value at significance level alpha
% crv1          = (right) critical value at significance level alpha
function [h,test,pval,crv0,crv1] = ks_t_test(sample,df,alpha,mctol,NMC)

mu=mean(sample);
s=std(sample)*sqrt((df-2)/df);
[sampleCDF,xCDF]=cdfcalc(sample);
nullCDF=tcdf((xCDF-mu)/s,df);
% Compute the test statistic of interest: T = max|S(x) - nullCDF(x)|.
delta1   = sampleCDF(1:end-1) - nullCDF;   % Vertical difference at jumps approaching from the LEFT.
delta2   = sampleCDF(2:end)   - nullCDF;   % Vertical difference at jumps approaching from the RIGHT.
deltaCDF = abs([delta1 ; delta2]);
test=max(deltaCDF);

if nargin < 2 || isempty(alpha)
   alpha = 0.05;
else
   if ~isscalar(alpha) || ~(0<alpha && alpha<1)
      error('Significance level must be between 0 and 1');
   end
end

if nargin <5 || isempty(NMC);
    if nargin < 4 || isempty(mctol)
        mctol = 10^(floor(log10(alpha/10)));
    else
        if ~isscalar(mctol) || mctol<=0
            error('Invalid Tolerance' );
        end
    end
    NMC=10^(ceil(log10(alpha*(1-alpha)/mctol^2)));
else
    if ~isscalar(NMC) || NMC<=0
        error('Invalid number of Monte Carlo Draws' );
    end
end

N=size(sample,1);

KSstatistics=NaN(NMC,1);

for j=1:NMC
    x=trnd(df,N,1);
    mu=mean(x);
    s=std(x)*sqrt((df-2)/df);
    [sampleCDF,xCDF]=cdfcalc(x);
    nullCDF=tcdf((xCDF-mu)/s,df);
    % Compute the test statistic of interest: T = max|S(x) - nullCDF(x)|.
    delta1   = sampleCDF(1:end-1) - nullCDF;   % Vertical difference at jumps approaching from the LEFT.
    delta2   = sampleCDF(2:end)   - nullCDF;   % Vertical difference at jumps approaching from the RIGHT.
    deltaCDF = abs([delta1 ; delta2]);

    KSstatistics(j) = max(deltaCDF);
end

crv0=quantile(KSstatistics,alpha);
crv1=quantile(KSstatistics,1-alpha);
pval = sum(KSstatistics>test)/NMC;
h = (test>crv1);
if sum(KSstatistics>test)<1
    disp(['No exceptions in ',num2str(NMC),' draws']);
end






