% function [Lt,pval]=levene3x(y,x)
% Levene test with sorting covariate using 1st and last thirds of the sample
% INPUTS:
% y(1:T,1)      = time-series to test equality ov varianes in 1st and last thirds
% x(1:T,1)      = sorting covariate (pass [] if not required)
% ct            = (optional) character indicator for the measure of central 
%                 tendency to use. Permitted values: 'mean', 'median', 
%                 'trimmean','mode'
% pct           = percentage of sample to trim (if ct='trimmean')
% OUTPUTS:
% Lt            = test statistic
% pval          = p-value corresponding to Lt
function [Lt,pval]=levene3x(y,x,ct,pct)

T=size(y,1);
if T<6
    error('Insufficient sample size')
end
df=T-1;
T0=floor(T/3);
if nargin<2 || isempty(x)
    % no sorting on covariate
    y1=y(1:T0);
    y3=y(T-T0+1:T);
else
    % sorting on covariate first
    [xs,ix]=sort(x);
    ys=y(ix);
    y1=ys(1:T0);
    y3=ys(T-T0+1:T);
end
if nargin<3
    ct0='mean';
else 
    ct0=ct;
end
if strcmp(ct0,'mean')
    z1=abs(y1-mean(y1));
    z3=abs(y3-mean(y3));
    m1=mean(z1);
    m3=mean(z3);
    m=mean([z1;z3]);
elseif strcmp(ct0,'median')
    z1=abs(y1-median(y1));
    z3=abs(y3-median(y3));
    m1=median(z1);
    m3=median(z3);
    m=median([z1;z3]);
elseif strcmp(ct0,'mode')
    z1=abs(y1-mode(y1));
    z3=abs(y3-mode(y3));
    m1=mode(z1);
    m3=mode(z3);
    m=mode([z1;z3]);    
elseif strcmp(ct0,'trimmean')
    if nargin<4
        pct0=10;
    else
        pct0=pct;
    end
    z1=abs(y1-trimmean(y1,pct0));
    z3=abs(y3-trimmean(y3,pct0));
    m1=trimmean(z1,pct0);
    m3=trimmean(z3,pct0);
    m=trimmean([z1;z3],pct0);
else
    error('Uknown measure of central tendency')
end
Lt=(T-2)*T0*((m1-m)^2+(m3-m)^2)/(sum((z1-m1).^2+(z3-m3).^2));
pval=min(fcdf(Lt,1,df),1-fcdf(Lt,1,df));
