% maximum likelihood estimator of hybrid piecewise elasticity of volatility
% model with normal innovations
% INPUTS:
% y(1:T)     = univariate data time-series
% alpha      = (optional) confidence level for estimates, defualt is 0.95
% start      = (optional) vector of starting guesses
% OUTPUTS:
% p(1:7) = estimated model parameters, namely
%   p(1)    = theta = long-run mean
%   p(2)    = kappa = speed of mean reversion
%   p(3)    = sigmabar = scale of volatility
%   p(4)    = gamma = elasticity of volatility at low levels
%   p(5)    = lambda = slope of return relationship for high levels
%   p(6)    = r_l = bound for low levels
%   p(7)    = r_u = upper bound of medium regime
%   p(8)    = nu  = degrees of freedom parameter
% ci(1:2,1:8)   = confidence bounds at significance level alpha
% aic       = Akaike information criterion
function [p,ci,aic] = hpev_t_mle(y,alpha,start)

if nargin<2
    alpha=0.05;
end
if nargin<3
    start=[mean(y),0,log(std(diff(y))),0.75,0.5,2,0,10];
end
T=size(y,1);

myoptions=statset('MaxFunEvals',10*T,'MaxIter',10000);


lb=[-Inf,-Inf,-10,-10,0,1,-10,1];
ub=Inf(1,8);ub(3)=0;ub(7)=log(10);


hpevt_loglikelihood=@(params,data,cens,freq)(-hpev_t_llf(params,data,cens,freq));
[p,ci]=mle(y, 'nloglf', hpevt_loglikelihood, 'start',start,'options',myoptions,'lowerbound',lb,'upperbound',ub,'optimfun','fmincon','alpha',alpha,'optimfun','fmincon');
aic=2*8+2*hpevt_loglikelihood(p,y,[],[]);
% constrain r_u>r_l by their differnce
p(7)=p(6)+exp(p(7));
ci(:,7)=ci(:,6)+exp(ci(:,7));
p(3)=exp(p(3));
ci(:,3)=exp(ci(:,3));
