! Compute critical values of KPSS tests for a large number of
! significance levels using very large number of Monte Carlo replications
! Including both trended and non-trended versions
!
! The code is copyright
! Sergei Morozov, Stanford University and TEU, 2012
! Initial f90 Implementation: 8/27/2012
! Last Change: 

! Variable Definitions
!________________________________
!________________________________
! Parameters
! NMC                = number of Monte Carlo Scenarios
! N                  = sample size
! p(:)               = vector of significance levels
!
! OUTPUTS
! CV(:,1:2)          = critical values at given significance levels (CV(:,1) no trend, CV(:,2) with trend)
!
! Auxiliary
! NP                 = size of vector p(:)
! kpss(1:NMC,1:2)    = simulated test statistics for both no-trend and with-trend cases
! Time0              = start time
! Time1              = finish time
! ic0,ic1,cr         = for timing purposes
! i                  = counter for Monte Carlo simulations
! j                  = counter for different significance levels
! t                  = counter for sample observations
! trend(1:N)         = vector 1,2,3,...,N
! b(1:2)             = regression coefficients of regression of y~N(0,1) on constant and linear trend
! s1, s2             = sum(y), sum([1:1:N]'.*y) used to calculate regression coefficients
! y(1:N)             = generated standard normals
! u(1:N)             = demeaned y (no-trend case) or regression residuals (trend case)
! S(1:N)             = cumulative sum of u
! brng               = underlying basic random number generator
! method             = method to get normal randoms from basic uniforms
! seed               = RNG seed
! errcode            = return error code from RNG
! info               = return code from sorting routine
! PN                 = vector of indices into simulated array corresponding to percentiles in p(:)
!
! SUBROUTINES USED
! CPU_TIME           = for timing purposes
! system_clock       = for timing purposes
! OMP_SET_NUM_PROCS  = to set the numer of processors in use in parallel loops
! vslnewstream       = create new stream of random numbers (from Intel MKL library)
! vdrnggaussian      = simulate double precision Gaussian random variates (from Intel MKL library)
! dlasrt             = sorting utility function from (MKL Lapack)
! 
! OUTPUT FILES
! critvalues_kpss.csv = store results in 3 columns: [significance,CV(:,1),CV(:,2)]

include 'mkl_vsl.f90'

program critvalue_kpss
  
  USE MKL_VSL_TYPE
  USE MKL_VSL
  include 'mkl_lasrt.f90'  

  ! Declarations
  !implicit none
  integer:: NMC,N,NP,i,j,t
  double precision, dimension(:), allocatable::y,u,S,trend
  double precision, dimension(:,:), allocatable::CV, kpss
  !double precision::CV(27,2), kpss(2000000000,2)
  integer,dimension(27)::PN
  double precision:: b(2), s1,s2,p(27)
  real:: Time0,Time1
  integer(4)::ic0,ic1,cr,errcode
  integer:: brng, method, seed, info
  TYPE (VSL_STREAM_STATE) :: stream
  
#ifdef _OPENMP
  call OMP_SET_NUM_THREADS(8)
#endif

  call CPU_TIME(Time0)
  call system_clock(count=ic0)


  NMC=4000000000_8 
  N=1000
  NP=27
  
  ! allocate
  allocate(CV(NP,2),kpss(NMC,2),trend(N))
  

  ! initialize
  s1=0.0d0
  s2=0.0d0
  y=0.0d0
  do t=1,N
     trend(t)=t
  enddo

  ! set significance levels of interest
  DATA p/1.0000E-06, 1.0000E-05, 1.0000E-04, 1.0000E-03, 2.5000E-03, 5.0000E-03, 1.0000E-02, 2.5000E-02, 5.0000E-02, &
       1.0000E-01, 2.0000E-01, 3.0000E-01, 4.0000E-01, 5.0000E-01, 6.0000E-01, 7.0000E-01, 8.0000E-01, 9.0000E-01, &
       9.5000E-01, 9.7500E-01, 9.9000E-01, 9.9500E-01, 9.9750E-01, 9.9900E-01, 9.99900E-01, 9.99990E-01, 9.99999E-01/
  ! set random number generator and seed
  brng = VSL_BRNG_MT19937
  method = VSL_METHOD_DGAUSSIAN_BOXMULLER2

  ! begin simulation
  errcode = vslnewstream(stream,brng,seed)
  !$omp parallel default(none)&
  !$omp shared (kpss,method,stream,N,trend,NMC) &
  !$omp private (i,t,errcode,y,u,S,s1,s2,b)
  !$omp do
  do i=1,NMC
     ! generate random numbers from standard normal distribution
     allocate(y(N),u(N),S(N))
     errcode = vdrnggaussian(method,stream,N,y,0.0d0,1.0d0)
     ! no-trend case
     ! demean
     u = y-sum(y)/N
     ! S=cumsum(u)
     S(1)=u(1)
     do t=2,N
        S(t)=S(t-1)+u(t)
     enddo

     ! kpss(i,1) = sum(S.^2)/N^2;
     kpss(i,1)=DOT_PRODUCT(S,S)/(dble(N)**2)
     ! linear trend case
     ! s1 = sum(y), s2=sum([1:1:N]'*.y)
     s1=sum(y)
     s2=DOT_PRODUCT(y,trend)
     ! closed form expressions for regression coefficients
     ! b(1)=(4*N+2)/(N*(N-1))*s1-6*s2/(N*(N-1));
     ! b(2)=-6*s1/(N*(N-1))+12/((N+1)*N*(N-1))*s2;
     b(1)=(4.0d0*dble(N)+2.0)/(dble(N)*(dble(N)-1.0d0))*s1-6.0d0*s2/(dble(N)*(dble(N)-1.0d0))
     b(2)=-6.0d0*s1/(dble(N)*(dble(N)-1.0d0))+12.0d0/((dble(N)+1.0d0)*dble(N)*(dble(N)-1.0d0))*s2
     ! regression residuals
     u = y-b(1)-b(2)*trend
     ! S=cumsum(u)
     S(1)=u(1)
     do t=2,N
        S(t)=S(t-1)+u(t)
     enddo
     ! kpss(i,2) = sum(S.^2)/N^2;
     kpss(i,2)=DOT_PRODUCT(S,S)/(dble(N)**2)
     deallocate(y,u,S)
  enddo
  !$omp end do
  !$omp end parallel

  ! sort kpss(:,1) and kpss(:,2)
  call dlasrt('I',NMC,kpss(:,1),info)
  call dlasrt('I',NMC,kpss(:,2),info)

  ! determine percentiles corresponding to significance levels in p(:)
  PN=max(IDINT((1-p)*NMC),1)
  CV=kpss(PN,:)

  call CPU_TIME(Time1)
  print *, 'Total Elapsed Time',Time1-Time0, ' CPU seconds'
  call system_clock(count=ic1,count_rate=cr)
  print *, 'Elapsed System Wall Time', dble(ic1-ic0)/dble(cr), 'System seconds'

  ! write critical values to the file
  open(file='critvalues_kpss.csv',unit=19,action="write")
  write(19,*) 'Significance Level, No-trend test critical value, Trend test critical value'
  do j=1,NP
     write(19,'(F9.6,",",F9.6,",",F9.6)') p(j),CV(j,1),CV(j,2)
  enddo
  close(unit=19,status="keep")
  errcode=vsldeletestream( stream )

end program critvalue_kpss




     





